float4x4 g_view_projection_matrix;
float4x4 g_world_matrix;
float4	g_light;
float4	g_camera_eye_vector;
float4	g_diffuse_mtrl;
float4	g_ambient_mtrl;
texture g_tex0;

float g_sinf = 0.0f;
float g_pow = 1.0f;

sampler2D tex0 : TEXUNIT0 = sampler_state
{
     Texture = (g_tex0);
    MIPFILTER = LINEAR;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
};

struct OutputVS
{
	float4 posH		: POSITION;
	float2 UV		: TEXCOORD0;
	float3 Light	: TEXCOORD1;
	float3 Norm		: TEXCOORD2;
	float3 View		: TEXCOORD3;
};

OutputVS SpecularVS( float4 posL : POSITION , float3 normal : NORMAL , float2 UV : TEXCOORD0 )
{
	OutputVS outVS = (OutputVS)0;

	outVS.Norm = mul( normal , g_world_matrix );
	
    float maxlerp = min(g_sinf, 1.0f);
	float ettlerp = g_sinf - 1.0f;

	float3 vl = lerp( float3(0, 0, 20.0f), posL.xyz, maxlerp );

	if(ettlerp > 0.0f) vl = lerp( vl, vl * 1.5f, sin(ettlerp));
	
	///////
	
	outVS.posH = mul( float4(vl, 1.0f) , g_world_matrix );

	if(g_pow <= 0.9f)
		outVS.Light = outVS.posH - float3(0.0f, -10.0f, -20.0f);  //g_light;
	else	
		outVS.Light = outVS.posH - float3(0.0f, 10.0f, 20.0f);  //g_light;

	outVS.posH = mul( outVS.posH , g_view_projection_matrix );
	
	outVS.UV = UV;	
	
	float4 posWorld = mul( vl , g_world_matrix );
	
	outVS.View = g_camera_eye_vector - posWorld;
	
	return outVS;
}

float4 SpecularPS( float2 UV : TEXCOORD0 , float3 Light : TEXCOORD1 , float3 Norm : TEXCOORD2 ,
				   float3 View : TEXCOORD3 ) : COLOR
{
	Norm	= normalize( Norm );
	Light	= normalize( Light );
	View	= normalize( View );
	
	//Get Texture
	float4 textureColor = tex2D( tex0 , UV );

	//Calculate diffuse color amount
	float Diff = dot(Norm, Light);
	
	//Specular
	float3 halfVec = normalize(View - Light);
	float Specular = pow(max( dot(Norm, halfVec), 0.0f), 32.0f);
	float SpecMat = float3(1.0f, 0.2f, 0.2f) * Specular;
	
	//Yes wierd return but looks nice ;D
	float4 result;
	if(g_pow <= 0.9f)
		result = ((g_diffuse_mtrl * 0.1f * Diff) + SpecMat + (textureColor * Diff));
	else
		result = ((g_diffuse_mtrl * Diff) + SpecMat + (textureColor * g_pow));
	
	result.w = 1.0f;
	return result;
}

technique normal
{
	pass p0
	{
		vertexShader = compile vs_2_0 SpecularVS();
		pixelShader = compile ps_2_0 SpecularPS();
	}
}